import { Component, OnInit, RendererFactory2, Renderer2 } from '@angular/core';
import { Title } from '@angular/platform-browser';
import { Router, ActivatedRouteSnapshot, NavigationEnd } from '@angular/router';
import { TranslateService, LangChangeEvent } from '@ngx-translate/core';
import dayjs from 'dayjs/esm';

import { AuthServerProvider } from 'app/core/auth/auth-jwt.service';
import { AccountService } from 'app/core/auth/account.service';
import { CookieService } from 'ngx-cookie-service';
import { MessageService, BroadCastMessage } from 'app/shared/service/message-service';

@Component({
  selector: 'jhi-main',
  templateUrl: './main.component.html',
  styleUrls: ['./main.component.scss'],
})
export class MainComponent implements OnInit {
  private renderer: Renderer2;

  activeMessages: Array<BroadCastMessage> = [];

  constructor(
    private accountService: AccountService,
    private titleService: Title,
    private router: Router,
    private translateService: TranslateService,
    private cookieService: CookieService,
    private authServerProvider: AuthServerProvider,
    private messageService: MessageService,
    rootRenderer: RendererFactory2
  ) {
    this.renderer = rootRenderer.createRenderer(document.querySelector('html'), null);
  }

  ngOnInit(): void {
    this.checkRequestToken();
    // try to log in automatically
    this.accountService.identity().subscribe();

    this.router.events.subscribe(event => {
      if (event instanceof NavigationEnd) {
        this.updateTitle();
      }
    });

    this.translateService.onLangChange.subscribe((langChangeEvent: LangChangeEvent) => {
      this.updateTitle();
      dayjs.locale(langChangeEvent.lang);
      this.renderer.setAttribute(document.querySelector('html'), 'lang', langChangeEvent.lang);
    });
  }

  private checkRequestToken(): void {
    const tokenCookie = this.cookieService.get('tempRequestToken');
    if (tokenCookie) {
      if (tokenCookie.length > 20)
        this.authServerProvider.refreshToken(tokenCookie, this.authServerProvider.getRememberMe()).subscribe({
          next: () => {
            this.accountService.identity(true).subscribe();
          },
          error: () => {
            //						 this.alertErrorComponent.addErrorAlert('OAuth2 authentication failed', 'Authentication failed')
          },
        });
    }
  }

  public getActiveMessages(): Array<BroadCastMessage> {
    this.messageService.getActiveMessages().subscribe(messages => (this.activeMessages = messages));
    return this.activeMessages;
  }

  private getPageTitle(routeSnapshot: ActivatedRouteSnapshot): string {
    const title: string = routeSnapshot.data['pageTitle'] ?? '';
    if (routeSnapshot.firstChild) {
      return this.getPageTitle(routeSnapshot.firstChild) || title;
    }
    return title;
  }

  private updateTitle(): void {
    let pageTitle = this.getPageTitle(this.router.routerState.snapshot.root);
    if (!pageTitle) {
      pageTitle = 'global.title';
    }
    this.translateService.get(pageTitle).subscribe(title => this.titleService.setTitle(title));
  }
}
