package at.ac.uibk.gitsearch.web.rest;

import static org.mockito.ArgumentMatchers.any;
import static org.mockito.Mockito.when;
import static org.springframework.security.test.web.servlet.request.SecurityMockMvcRequestPostProcessors.csrf;
import static org.springframework.test.web.servlet.request.MockMvcRequestBuilders.delete;
import static org.springframework.test.web.servlet.request.MockMvcRequestBuilders.get;
import static org.springframework.test.web.servlet.request.MockMvcRequestBuilders.post;
import static org.springframework.test.web.servlet.result.MockMvcResultMatchers.status;

import at.ac.uibk.gitsearch.domain.ChatMessage;
import at.ac.uibk.gitsearch.domain.Review;
import at.ac.uibk.gitsearch.domain.User;
import at.ac.uibk.gitsearch.security.AuthoritiesConstants;
import at.ac.uibk.gitsearch.service.ChatMessageService;
import at.ac.uibk.gitsearch.service.ReviewService;
import at.ac.uibk.gitsearch.service.UserService;
import java.util.Collections;
import java.util.Optional;
import javax.transaction.Transactional;
import org.junit.jupiter.api.Test;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.autoconfigure.web.servlet.AutoConfigureMockMvc;
import org.springframework.boot.test.context.SpringBootTest;
import org.springframework.boot.test.mock.mockito.MockBean;
import org.springframework.http.MediaType;
import org.springframework.security.test.context.support.WithMockUser;
import org.springframework.test.web.servlet.MockMvc;

@SpringBootTest
@AutoConfigureMockMvc
class ChatMessageResourceIT {

    @Autowired
    private MockMvc mockMvc;

    @MockBean
    private ChatMessageService chatMessageService;

    @MockBean
    private UserService userService;

    @MockBean
    private ReviewService reviewService;

    // Sample Mock Data
    private ChatMessage mockChatMessage = new ChatMessage(); // Initialize with sample data
    private Review mockReview = new Review(); // Initialize with sample data
    private User mockUser = new User(); // Initialize with sample data

    @Test
    @WithMockUser(roles = "ADMIN")
    void testGetAllChatMessages() throws Exception {
        when(chatMessageService.getAllChatMessages()).thenReturn(Collections.singletonList(mockChatMessage));

        mockMvc.perform(get("/api/chatmessages")).andExpect(status().isOk());
    }

    @Test
    @WithMockUser(roles = "ADMIN")
    void testGetChatMessageById() throws Exception {
        when(chatMessageService.getChatMessageById(1L)).thenReturn(Optional.of(mockChatMessage));

        mockMvc.perform(get("/api/chatmessages/{id}", 1L)).andExpect(status().isOk());
    }

    @Test
    @WithMockUser(roles = "USER")
    void testSaveChatMessage() throws Exception {
        when(chatMessageService.saveChatMessage(any())).thenReturn(mockChatMessage);

        mockMvc
            .perform(post("/api/chatmessages").with(csrf()).contentType(MediaType.APPLICATION_JSON).content("{}")) // Provide a valid JSON representation of a ChatMessageDTO
            .andExpect(status().isOk());
    }

    @Test
    @WithMockUser(authorities = AuthoritiesConstants.ADMIN, username = "admin")
    void testDeleteChatMessage() throws Exception {
        mockMvc.perform(delete("/api/chatmessages/{id}", 1L).with(csrf())).andExpect(status().isOk());
    }

    @Test
    @WithMockUser(roles = "USER")
    void testFindByReview() throws Exception {
        when(reviewService.getReviewById(1L)).thenReturn(mockReview);
        when(userService.getUserWithAuthorities()).thenReturn(Optional.of(mockUser));
        when(chatMessageService.hasAccessToReview(mockUser, mockReview)).thenReturn(true);
        when(chatMessageService.findByReview(mockReview)).thenReturn(Collections.singletonList(mockChatMessage));

        mockMvc.perform(get("/api/chatmessages/review/{reviewId}", 1L)).andExpect(status().isOk());
    }

    @Test
    @WithMockUser(roles = "USER")
    @Transactional
    void testFindByReviewOrderByTimestampAsc() throws Exception {
        when(reviewService.getReviewById(1L)).thenReturn(mockReview);
        when(userService.getUserWithAuthorities()).thenReturn(Optional.of(mockUser));
        when(chatMessageService.hasAccessToReview(mockUser, mockReview)).thenReturn(true);
        when(chatMessageService.findByReviewOrderByTimestampAsc(mockReview)).thenReturn(Collections.singletonList(mockChatMessage));

        mockMvc.perform(get("/api/chatmessages/review/{reviewId}/ordered", 1L).with(csrf())).andExpect(status().isOk());
    }
}
