package at.ac.uibk.gitsearch.web.rest;

import static org.assertj.core.api.Assertions.assertThat;
import static org.hamcrest.Matchers.hasItem;
import static org.springframework.security.test.web.servlet.request.SecurityMockMvcRequestPostProcessors.csrf;
import static org.springframework.test.web.servlet.request.MockMvcRequestBuilders.delete;
import static org.springframework.test.web.servlet.request.MockMvcRequestBuilders.get;
import static org.springframework.test.web.servlet.request.MockMvcRequestBuilders.patch;
import static org.springframework.test.web.servlet.request.MockMvcRequestBuilders.post;
import static org.springframework.test.web.servlet.request.MockMvcRequestBuilders.put;
import static org.springframework.test.web.servlet.result.MockMvcResultMatchers.content;
import static org.springframework.test.web.servlet.result.MockMvcResultMatchers.jsonPath;
import static org.springframework.test.web.servlet.result.MockMvcResultMatchers.status;

import at.ac.uibk.gitsearch.IntegrationTest;
import at.ac.uibk.gitsearch.domain.SavedSearches;
import at.ac.uibk.gitsearch.domain.User;
import at.ac.uibk.gitsearch.repository.jpa.SavedSearchesRepository;
import at.ac.uibk.gitsearch.service.dto.SavedSearchesDTO;
import at.ac.uibk.gitsearch.service.mapper.SavedSearchesMapper;
import java.util.List;
import java.util.Random;
import java.util.concurrent.atomic.AtomicLong;
import javax.persistence.EntityManager;
import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Test;
import org.junit.jupiter.api.extension.ExtendWith;
import org.mockito.junit.jupiter.MockitoExtension;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.autoconfigure.web.servlet.AutoConfigureMockMvc;
import org.springframework.http.MediaType;
import org.springframework.security.test.context.support.WithMockUser;
import org.springframework.test.web.servlet.MockMvc;
import org.springframework.transaction.annotation.Transactional;

/**
 * Integration tests for the {@link SavedSearchesResource} REST controller.
 */
@IntegrationTest
@ExtendWith(MockitoExtension.class)
@AutoConfigureMockMvc
@WithMockUser
class SavedSearchesResourceIT {

    private static final String DEFAULT_NAME = "AAAAAAAAAA";
    private static final String UPDATED_NAME = "BBBBBBBBBB";

    private static final String DEFAULT_JSON_QUERY = "AAAAAAAAAA";
    private static final String UPDATED_JSON_QUERY = "BBBBBBBBBB";

    private static final String ENTITY_API_URL = "/api/saved-searches";
    private static final String ENTITY_API_URL_ID = ENTITY_API_URL + "/{id}";

    @SuppressWarnings("unused")
    private static final String ENTITY_SEARCH_API_URL = "/api/_search/saved-searches";

    private static Random random = new Random();
    private static AtomicLong count = new AtomicLong(random.nextInt() + (2 * Integer.MAX_VALUE));

    @Autowired
    private SavedSearchesRepository savedSearchesRepository;

    @Autowired
    private SavedSearchesMapper savedSearchesMapper;

    @Autowired
    private EntityManager em;

    @Autowired
    private MockMvc restSavedSearchesMockMvc;

    private SavedSearches savedSearches;

    /**
     * Create an entity for this test.
     *
     * This is a static method, as tests for other entities might also need it,
     * if they test an entity which requires the current entity.
     */
    public static SavedSearches createEntity(EntityManager em) {
        SavedSearches savedSearches = new SavedSearches().name(DEFAULT_NAME).jsonQuery(DEFAULT_JSON_QUERY);
        // Add required entity
        User user = UserResourceIT.createEntity(em);
        em.persist(user);
        em.flush();
        savedSearches.setUser(user);
        return savedSearches;
    }

    /**
     * Create an updated entity for this test.
     *
     * This is a static method, as tests for other entities might also need it,
     * if they test an entity which requires the current entity.
     */
    public static SavedSearches createUpdatedEntity(EntityManager em) {
        SavedSearches savedSearches = new SavedSearches().name(UPDATED_NAME).jsonQuery(UPDATED_JSON_QUERY);
        // Add required entity
        User user = UserResourceIT.createEntity(em);
        em.persist(user);
        em.flush();
        savedSearches.setUser(user);
        return savedSearches;
    }

    @BeforeEach
    public void initTest() {
        savedSearches = createEntity(em);
    }

    @Test
    @Transactional
    void createSavedSearches() throws Exception {
        int databaseSizeBeforeCreate = savedSearchesRepository.findAll().size();
        // Create the SavedSearches
        SavedSearchesDTO savedSearchesDTO = savedSearchesMapper.toDto(savedSearches);
        restSavedSearchesMockMvc
            .perform(
                post(ENTITY_API_URL)
                    .contentType(MediaType.APPLICATION_JSON)
                    .content(TestUtil.convertObjectToJsonBytes(savedSearchesDTO))
                    .with(csrf().asHeader())
            )
            .andExpect(status().isCreated());

        // Validate the SavedSearches in the database
        List<SavedSearches> savedSearchesList = savedSearchesRepository.findAll();
        assertThat(savedSearchesList).hasSize(databaseSizeBeforeCreate + 1);
        SavedSearches testSavedSearches = savedSearchesList.get(savedSearchesList.size() - 1);
        assertThat(testSavedSearches.getName()).isEqualTo(DEFAULT_NAME);
        assertThat(testSavedSearches.getJsonQuery()).isEqualTo(DEFAULT_JSON_QUERY);
    }

    @Test
    @Transactional
    void createSavedSearchesWithExistingId() throws Exception {
        // Create the SavedSearches with an existing ID
        savedSearches.setId(1L);
        SavedSearchesDTO savedSearchesDTO = savedSearchesMapper.toDto(savedSearches);

        int databaseSizeBeforeCreate = savedSearchesRepository.findAll().size();

        // An entity with an existing ID cannot be created, so this API call must fail
        restSavedSearchesMockMvc
            .perform(
                post(ENTITY_API_URL)
                    .contentType(MediaType.APPLICATION_JSON)
                    .content(TestUtil.convertObjectToJsonBytes(savedSearchesDTO))
                    .with(csrf().asHeader())
            )
            .andExpect(status().isBadRequest());

        // Validate the SavedSearches in the database
        List<SavedSearches> savedSearchesList = savedSearchesRepository.findAll();
        assertThat(savedSearchesList).hasSize(databaseSizeBeforeCreate);
    }

    @Test
    @Transactional
    void checkNameIsRequired() throws Exception {
        int databaseSizeBeforeTest = savedSearchesRepository.findAll().size();
        // set the field null
        savedSearches.setName(null);

        // Create the SavedSearches, which fails.
        SavedSearchesDTO savedSearchesDTO = savedSearchesMapper.toDto(savedSearches);

        restSavedSearchesMockMvc
            .perform(
                post(ENTITY_API_URL)
                    .contentType(MediaType.APPLICATION_JSON)
                    .content(TestUtil.convertObjectToJsonBytes(savedSearchesDTO))
                    .with(csrf().asHeader())
            )
            .andExpect(status().isBadRequest());

        List<SavedSearches> savedSearchesList = savedSearchesRepository.findAll();
        assertThat(savedSearchesList).hasSize(databaseSizeBeforeTest);
    }

    @Test
    @Transactional
    void checkJsonQueryIsRequired() throws Exception {
        int databaseSizeBeforeTest = savedSearchesRepository.findAll().size();
        // set the field null
        savedSearches.setJsonQuery(null);

        // Create the SavedSearches, which fails.
        SavedSearchesDTO savedSearchesDTO = savedSearchesMapper.toDto(savedSearches);

        restSavedSearchesMockMvc
            .perform(
                post(ENTITY_API_URL)
                    .contentType(MediaType.APPLICATION_JSON)
                    .content(TestUtil.convertObjectToJsonBytes(savedSearchesDTO))
                    .with(csrf().asHeader())
            )
            .andExpect(status().isBadRequest());

        List<SavedSearches> savedSearchesList = savedSearchesRepository.findAll();
        assertThat(savedSearchesList).hasSize(databaseSizeBeforeTest);
    }

    @Test
    @Transactional
    void getAllSavedSearches() throws Exception {
        // Initialize the database
        savedSearchesRepository.saveAndFlush(savedSearches);

        // Get all the savedSearchesList
        restSavedSearchesMockMvc
            .perform(get(ENTITY_API_URL + "?sort=id,desc"))
            .andExpect(status().isOk())
            .andExpect(content().contentType(MediaType.APPLICATION_JSON_VALUE))
            .andExpect(jsonPath("$.[*].id").value(hasItem(savedSearches.getId().intValue())))
            .andExpect(jsonPath("$.[*].name").value(hasItem(DEFAULT_NAME)))
            .andExpect(jsonPath("$.[*].jsonQuery").value(hasItem(DEFAULT_JSON_QUERY)));
    }

    @Test
    @Transactional
    void getSavedSearches() throws Exception {
        // Initialize the database
        savedSearchesRepository.saveAndFlush(savedSearches);

        // Get the savedSearches
        restSavedSearchesMockMvc
            .perform(get(ENTITY_API_URL_ID, savedSearches.getId()))
            .andExpect(status().isOk())
            .andExpect(content().contentType(MediaType.APPLICATION_JSON_VALUE))
            .andExpect(jsonPath("$.id").value(savedSearches.getId().intValue()))
            .andExpect(jsonPath("$.name").value(DEFAULT_NAME))
            .andExpect(jsonPath("$.jsonQuery").value(DEFAULT_JSON_QUERY));
    }

    @Test
    @Transactional
    void getNonExistingSavedSearches() throws Exception {
        // Get the savedSearches
        restSavedSearchesMockMvc.perform(get(ENTITY_API_URL_ID, Long.MAX_VALUE)).andExpect(status().isNotFound());
    }

    @Test
    @Transactional
    void putNewSavedSearches() throws Exception {
        // Initialize the database
        savedSearchesRepository.saveAndFlush(savedSearches);

        int databaseSizeBeforeUpdate = savedSearchesRepository.findAll().size();

        // Update the savedSearches
        SavedSearches updatedSavedSearches = savedSearchesRepository.findById(savedSearches.getId()).get();
        // Disconnect from session so that the updates on updatedSavedSearches are not directly saved in db
        em.detach(updatedSavedSearches);
        updatedSavedSearches.name(UPDATED_NAME).jsonQuery(UPDATED_JSON_QUERY);
        SavedSearchesDTO savedSearchesDTO = savedSearchesMapper.toDto(updatedSavedSearches);

        restSavedSearchesMockMvc
            .perform(
                put(ENTITY_API_URL_ID, savedSearchesDTO.getId())
                    .contentType(MediaType.APPLICATION_JSON)
                    .content(TestUtil.convertObjectToJsonBytes(savedSearchesDTO))
                    .with(csrf().asHeader())
            )
            .andExpect(status().isOk());

        // Validate the SavedSearches in the database
        List<SavedSearches> savedSearchesList = savedSearchesRepository.findAll();
        assertThat(savedSearchesList).hasSize(databaseSizeBeforeUpdate);
        SavedSearches testSavedSearches = savedSearchesList.get(savedSearchesList.size() - 1);
        assertThat(testSavedSearches.getName()).isEqualTo(UPDATED_NAME);
        assertThat(testSavedSearches.getJsonQuery()).isEqualTo(UPDATED_JSON_QUERY);
    }

    @Test
    @Transactional
    void putNonExistingSavedSearches() throws Exception {
        int databaseSizeBeforeUpdate = savedSearchesRepository.findAll().size();
        savedSearches.setId(count.incrementAndGet());

        // Create the SavedSearches
        SavedSearchesDTO savedSearchesDTO = savedSearchesMapper.toDto(savedSearches);

        // If the entity doesn't have an ID, it will throw BadRequestAlertException
        restSavedSearchesMockMvc
            .perform(
                put(ENTITY_API_URL_ID, savedSearchesDTO.getId())
                    .contentType(MediaType.APPLICATION_JSON)
                    .content(TestUtil.convertObjectToJsonBytes(savedSearchesDTO))
                    .with(csrf().asHeader())
            )
            .andExpect(status().isBadRequest());

        // Validate the SavedSearches in the database
        List<SavedSearches> savedSearchesList = savedSearchesRepository.findAll();
        assertThat(savedSearchesList).hasSize(databaseSizeBeforeUpdate);
    }

    @Test
    @Transactional
    void putWithIdMismatchSavedSearches() throws Exception {
        int databaseSizeBeforeUpdate = savedSearchesRepository.findAll().size();
        savedSearches.setId(count.incrementAndGet());

        // Create the SavedSearches
        SavedSearchesDTO savedSearchesDTO = savedSearchesMapper.toDto(savedSearches);

        // If url ID doesn't match entity ID, it will throw BadRequestAlertException
        restSavedSearchesMockMvc
            .perform(
                put(ENTITY_API_URL_ID, count.incrementAndGet())
                    .contentType(MediaType.APPLICATION_JSON)
                    .content(TestUtil.convertObjectToJsonBytes(savedSearchesDTO))
                    .with(csrf().asHeader())
            )
            .andExpect(status().isBadRequest());

        // Validate the SavedSearches in the database
        List<SavedSearches> savedSearchesList = savedSearchesRepository.findAll();
        assertThat(savedSearchesList).hasSize(databaseSizeBeforeUpdate);
    }

    @Test
    @Transactional
    void putWithMissingIdPathParamSavedSearches() throws Exception {
        int databaseSizeBeforeUpdate = savedSearchesRepository.findAll().size();
        savedSearches.setId(count.incrementAndGet());

        // Create the SavedSearches
        SavedSearchesDTO savedSearchesDTO = savedSearchesMapper.toDto(savedSearches);

        // If url ID doesn't match entity ID, it will throw BadRequestAlertException
        restSavedSearchesMockMvc
            .perform(
                put(ENTITY_API_URL)
                    .contentType(MediaType.APPLICATION_JSON)
                    .content(TestUtil.convertObjectToJsonBytes(savedSearchesDTO))
                    .with(csrf().asHeader())
            )
            .andExpect(status().isMethodNotAllowed());

        // Validate the SavedSearches in the database
        List<SavedSearches> savedSearchesList = savedSearchesRepository.findAll();
        assertThat(savedSearchesList).hasSize(databaseSizeBeforeUpdate);
    }

    @Test
    @Transactional
    void partialUpdateSavedSearchesWithPatch() throws Exception {
        // Initialize the database
        savedSearchesRepository.saveAndFlush(savedSearches);

        int databaseSizeBeforeUpdate = savedSearchesRepository.findAll().size();

        // Update the savedSearches using partial update
        SavedSearches partialUpdatedSavedSearches = new SavedSearches();
        partialUpdatedSavedSearches.setId(savedSearches.getId());

        partialUpdatedSavedSearches.name(UPDATED_NAME).jsonQuery(UPDATED_JSON_QUERY);

        restSavedSearchesMockMvc
            .perform(
                patch(ENTITY_API_URL_ID, partialUpdatedSavedSearches.getId())
                    .contentType("application/merge-patch+json")
                    .content(TestUtil.convertObjectToJsonBytes(partialUpdatedSavedSearches))
                    .with(csrf().asHeader())
            )
            .andExpect(status().isOk());

        // Validate the SavedSearches in the database
        List<SavedSearches> savedSearchesList = savedSearchesRepository.findAll();
        assertThat(savedSearchesList).hasSize(databaseSizeBeforeUpdate);
        SavedSearches testSavedSearches = savedSearchesList.get(savedSearchesList.size() - 1);
        assertThat(testSavedSearches.getName()).isEqualTo(UPDATED_NAME);
        assertThat(testSavedSearches.getJsonQuery()).isEqualTo(UPDATED_JSON_QUERY);
    }

    @Test
    @Transactional
    void fullUpdateSavedSearchesWithPatch() throws Exception {
        // Initialize the database
        savedSearchesRepository.saveAndFlush(savedSearches);

        int databaseSizeBeforeUpdate = savedSearchesRepository.findAll().size();

        // Update the savedSearches using partial update
        SavedSearches partialUpdatedSavedSearches = new SavedSearches();
        partialUpdatedSavedSearches.setId(savedSearches.getId());

        partialUpdatedSavedSearches.name(UPDATED_NAME).jsonQuery(UPDATED_JSON_QUERY);

        restSavedSearchesMockMvc
            .perform(
                patch(ENTITY_API_URL_ID, partialUpdatedSavedSearches.getId())
                    .contentType("application/merge-patch+json")
                    .content(TestUtil.convertObjectToJsonBytes(partialUpdatedSavedSearches))
                    .with(csrf().asHeader())
            )
            .andExpect(status().isOk());

        // Validate the SavedSearches in the database
        List<SavedSearches> savedSearchesList = savedSearchesRepository.findAll();
        assertThat(savedSearchesList).hasSize(databaseSizeBeforeUpdate);
        SavedSearches testSavedSearches = savedSearchesList.get(savedSearchesList.size() - 1);
        assertThat(testSavedSearches.getName()).isEqualTo(UPDATED_NAME);
        assertThat(testSavedSearches.getJsonQuery()).isEqualTo(UPDATED_JSON_QUERY);
    }

    @Test
    @Transactional
    void patchNonExistingSavedSearches() throws Exception {
        int databaseSizeBeforeUpdate = savedSearchesRepository.findAll().size();
        savedSearches.setId(count.incrementAndGet());

        // Create the SavedSearches
        SavedSearchesDTO savedSearchesDTO = savedSearchesMapper.toDto(savedSearches);

        // If the entity doesn't have an ID, it will throw BadRequestAlertException
        restSavedSearchesMockMvc
            .perform(
                patch(ENTITY_API_URL_ID, savedSearchesDTO.getId())
                    .contentType("application/merge-patch+json")
                    .content(TestUtil.convertObjectToJsonBytes(savedSearchesDTO))
                    .with(csrf().asHeader())
            )
            .andExpect(status().isBadRequest());

        // Validate the SavedSearches in the database
        List<SavedSearches> savedSearchesList = savedSearchesRepository.findAll();
        assertThat(savedSearchesList).hasSize(databaseSizeBeforeUpdate);
    }

    @Test
    @Transactional
    void patchWithIdMismatchSavedSearches() throws Exception {
        int databaseSizeBeforeUpdate = savedSearchesRepository.findAll().size();
        savedSearches.setId(count.incrementAndGet());

        // Create the SavedSearches
        SavedSearchesDTO savedSearchesDTO = savedSearchesMapper.toDto(savedSearches);

        // If url ID doesn't match entity ID, it will throw BadRequestAlertException
        restSavedSearchesMockMvc
            .perform(
                patch(ENTITY_API_URL_ID, count.incrementAndGet())
                    .contentType("application/merge-patch+json")
                    .content(TestUtil.convertObjectToJsonBytes(savedSearchesDTO))
                    .with(csrf().asHeader())
            )
            .andExpect(status().isBadRequest());

        // Validate the SavedSearches in the database
        List<SavedSearches> savedSearchesList = savedSearchesRepository.findAll();
        assertThat(savedSearchesList).hasSize(databaseSizeBeforeUpdate);
    }

    @Test
    @Transactional
    void patchWithMissingIdPathParamSavedSearches() throws Exception {
        int databaseSizeBeforeUpdate = savedSearchesRepository.findAll().size();
        savedSearches.setId(count.incrementAndGet());

        // Create the SavedSearches
        SavedSearchesDTO savedSearchesDTO = savedSearchesMapper.toDto(savedSearches);

        // If url ID doesn't match entity ID, it will throw BadRequestAlertException
        restSavedSearchesMockMvc
            .perform(
                patch(ENTITY_API_URL)
                    .contentType("application/merge-patch+json")
                    .content(TestUtil.convertObjectToJsonBytes(savedSearchesDTO))
                    .with(csrf().asHeader())
            )
            .andExpect(status().isMethodNotAllowed());

        // Validate the SavedSearches in the database
        List<SavedSearches> savedSearchesList = savedSearchesRepository.findAll();
        assertThat(savedSearchesList).hasSize(databaseSizeBeforeUpdate);
    }

    @Test
    @Transactional
    void deleteSavedSearches() throws Exception {
        // Initialize the database
        savedSearchesRepository.saveAndFlush(savedSearches);

        int databaseSizeBeforeDelete = savedSearchesRepository.findAll().size();

        // Delete the savedSearches
        restSavedSearchesMockMvc
            .perform(delete(ENTITY_API_URL_ID, savedSearches.getId()).accept(MediaType.APPLICATION_JSON).with(csrf().asHeader()))
            .andExpect(status().isNoContent());

        // Validate the database contains one less item
        List<SavedSearches> savedSearchesList = savedSearchesRepository.findAll();
        assertThat(savedSearchesList).hasSize(databaseSizeBeforeDelete - 1);
    }
}
